@@ -555,36 +555,43 @@ void gl_solver(int *restrict x, double *restrict z, double *restrict y, int *res
   free(gradientOld);
 }
 
-SEXP R_gl_solver(SEXP R_x, SEXP R_z, SEXP R_y, SEXP R_nRows, SEXP R_intercept, SEXP R_beta, SEXP R_residual, SEXP R_linear, SEXP R_numLevels, SEXP R_nVars, SEXP R_catIndices, SEXP R_contIndices, SEXP R_catcatIndices, SEXP R_contcontIndices, SEXP R_catcontIndices, SEXP R_lambda, SEXP R_tol, SEXP R_alpha, SEXP R_maxIter, SEXP R_convergedFlag, SEXP R_objValue, SEXP R_steps, SEXP R_family){
-  PROTECT(R_x = coerceVector(duplicate(R_x), INTSXP));
-  PROTECT(R_z = coerceVector(duplicate(R_z), REALSXP));
-  PROTECT(R_y = coerceVector(duplicate(R_y), REALSXP));
-  PROTECT(R_nRows = coerceVector(duplicate(R_nRows), INTSXP));
-  PROTECT(R_intercept = coerceVector(duplicate(R_intercept), REALSXP));
-  PROTECT(R_beta = coerceVector(duplicate(R_beta), REALSXP));
-  PROTECT(R_residual = coerceVector(duplicate(R_residual), REALSXP));
-  PROTECT(R_linear = coerceVector(duplicate(R_linear), REALSXP));
-  PROTECT(R_numLevels = coerceVector(duplicate(R_numLevels), INTSXP));
-  PROTECT(R_nVars = coerceVector(duplicate(R_nVars), INTSXP));
-  PROTECT(R_catIndices = coerceVector(duplicate(R_catIndices), INTSXP));
-  PROTECT(R_contIndices = coerceVector(duplicate(R_contIndices), INTSXP));
-  PROTECT(R_catcatIndices = coerceVector(duplicate(R_catcatIndices), INTSXP));
-  PROTECT(R_contcontIndices = coerceVector(duplicate(R_contcontIndices), INTSXP));
-  PROTECT(R_catcontIndices = coerceVector(duplicate(R_catcontIndices), INTSXP));
-  PROTECT(R_lambda = coerceVector(duplicate(R_lambda), REALSXP));
-  PROTECT(R_tol = coerceVector(duplicate(R_tol), REALSXP));
-  PROTECT(R_alpha = coerceVector(duplicate(R_alpha), REALSXP));
-  PROTECT(R_maxIter = coerceVector(duplicate(R_maxIter), INTSXP));
-  PROTECT(R_convergedFlag = coerceVector(duplicate(R_convergedFlag), INTSXP));
-  PROTECT(R_objValue = coerceVector(duplicate(R_objValue), REALSXP));
-  PROTECT(R_steps = coerceVector(duplicate(R_steps), REALSXP));
-  PROTECT(R_family = coerceVector(duplicate(R_family), INTSXP));
+SEXP R_gl_solver(SEXP R_x, SEXP R_z, SEXP R_y, SEXP R_nRows, SEXP R_intercept, SEXP R_beta, SEXP R_numLevels, SEXP R_nVars, SEXP R_catIndices, SEXP R_contIndices, SEXP R_catcatIndices, SEXP R_contcontIndices, SEXP R_catcontIndices, SEXP R_lambda, SEXP R_tol, SEXP R_alpha, SEXP R_maxIter, SEXP R_family){
+  PROTECT(R_x = coerceVector(R_x, INTSXP));
+  PROTECT(R_z = coerceVector(R_z, REALSXP));
+  PROTECT(R_y = coerceVector(R_y, REALSXP));
+  PROTECT(R_nRows = coerceVector(R_nRows, INTSXP));
+  PROTECT(R_intercept = coerceVector(R_intercept, REALSXP));
+  SEXP R_interceptCopy = duplicate(R_intercept);
+  PROTECT(R_beta = coerceVector(R_beta, REALSXP));
+  SEXP R_betaCopy = duplicate(R_beta);
+  SEXP R_residual = PROTECT(allocVector(REALSXP, asInteger(R_nRows)));
+  memset(REAL(R_residual), 0, asInteger(R_nRows)*sizeof(double));
+  SEXP R_linear = PROTECT(allocVector(REALSXP, asInteger(R_nRows)));
+  memset(REAL(R_linear), 0, asInteger(R_nRows)*sizeof(double));
+  PROTECT(R_numLevels = coerceVector(R_numLevels, INTSXP));
+  PROTECT(R_nVars = coerceVector(R_nVars, INTSXP));
+  PROTECT(R_catIndices = coerceVector(R_catIndices, INTSXP));
+  PROTECT(R_contIndices = coerceVector(R_contIndices, INTSXP));
+  PROTECT(R_catcatIndices = coerceVector(R_catcatIndices, INTSXP));
+  PROTECT(R_contcontIndices = coerceVector(R_contcontIndices, INTSXP));
+  PROTECT(R_catcontIndices = coerceVector(R_catcontIndices, INTSXP));
+  PROTECT(R_lambda = coerceVector(R_lambda, REALSXP));
+  PROTECT(R_tol = coerceVector(R_tol, REALSXP));
+  PROTECT(R_alpha = coerceVector(R_alpha, REALSXP));
+  PROTECT(R_maxIter = coerceVector(R_maxIter, INTSXP));
+  SEXP R_convergedFlag = PROTECT(allocVector(INTSXP, 1));
+  memset(INTEGER(R_convergedFlag), 0, sizeof(int));
+  SEXP R_objValue = PROTECT(allocVector(REALSXP, 1));
+  memset(REAL(R_objValue), 0, sizeof(double));
+  SEXP R_steps = PROTECT(allocVector(REALSXP, asInteger(R_maxIter)));
+  memset(REAL(R_steps), 0, asInteger(R_maxIter)*sizeof(double));
+  PROTECT(R_family = coerceVector(R_family, INTSXP));
   int *restrict x = INTEGER(R_x);
   double *restrict z = REAL(R_z);
   double *restrict y = REAL(R_y);
   int *restrict nRows = INTEGER(R_nRows);
-  double *restrict intercept = REAL(R_intercept);
-  double *restrict beta = REAL(R_beta);
+  double *restrict intercept = REAL(R_interceptCopy);
+  double *restrict beta = REAL(R_betaCopy);
   double *restrict residual = REAL(R_residual);
   double *restrict linear = REAL(R_linear);
   int *restrict numLevels = INTEGER(R_numLevels);
@@ -605,8 +612,8 @@ SEXP R_gl_solver(SEXP R_x, SEXP R_z, SEXP R_y, SEXP R_nRows, SEXP R_intercept, S
   gl_solver(x, z, y, nRows, intercept, beta, residual, linear, numLevels, nVars, catIndices, contIndices, catcatIndices, contcontIndices, catcontIndices, lambda, tol, alpha, maxIter, convergedFlag, objValue, steps, family);
   UNPROTECT(23);
   SEXP result = PROTECT(allocVector(VECSXP, 4));
-  SET_VECTOR_ELT(result, 0, R_intercept);
-  SET_VECTOR_ELT(result, 1, R_beta);
+  SET_VECTOR_ELT(result, 0, R_interceptCopy);
+  SET_VECTOR_ELT(result, 1, R_betaCopy);
   SET_VECTOR_ELT(result, 2, R_residual);
   SET_VECTOR_ELT(result, 3, R_objValue);
   const char *names[4] = {"mu", "coefficients", "res", "objValue"};
